/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

#ifndef MOBIES_UDMDOM_H
#define MOBIES_UDMDOM_H

#ifndef MOBIES_UDMBASE_H
#include <UdmBase.h>
#endif

//these predefinitions are needed for typedefs
class DOM_Node;
class DOM_Element;
class DOMString;

typedef map<unsigned long, DOM_Element> IdToDomElementMap;
typedef pair<const unsigned long, DOM_Element> IdToDomElementMapItem;

	
namespace UdmDom
{
	typedef Udm::Object Object;

	//check if an input XML is in UDM UML or XMI UML format
	//(used mainly by  Udm.exe)

	//these will be linked as an external .cpp file.
	
	
	bool isXmi(const string& filename);

	
	class DomDataNetwork : public Udm::DataNetwork {
		static bool multiroles;
		static int instancecount;
		string savesystemname;
		bool saveondestroy;
	public:
		static string DTDPath;
		//meta type-cache-by-name
		map<string, const Uml::Class> meta_class_cache;

		DomDataNetwork(const Udm::UdmDiagram &metainfo);
		~DomDataNetwork();
		void CreateNew(const string &systemname, 
									const string &metalocator, const Uml::Class &rootclass, 
									enum Udm::BackendSemantics sem = Udm::CHANGES_PERSIST_ALWAYS);
		void OpenExisting(const string &systemname, 
									const string &metalocator, 
									enum Udm::BackendSemantics sem = Udm::CHANGES_PERSIST_ALWAYS);
		void MapExistingIDs(DOM_Node &d);
		void SaveAs(string systemname); 
		void CloseWithUpdate();
		void CloseNoUpdate();

		void CommitEditSequence();
		void AbortEditSequence();

// if MultiRoles is enabled, a child object can assume 
//		multiple roles in its container simultaneously (but always at least one)
// otherwise, a new role assignment removes prior role 
		static bool MultiRolesEnabled() { return multiroles; }
		static bool EnableMultiRoles(bool p) { multiroles = p; }

		virtual Object ObjectById(Object::uniqueId_type id);
		
		static Udm::DataNetwork *factory(const Udm::UdmDiagram &meta) 
		{
			return new DomDataNetwork(meta);
		}
	
		static const string FindFile(const string file_name);
		
		//map of Elements in the Diagram
		IdToDomElementMap DomElements;
		void DoMapping(const DOM_Element &e, long id,  bool force);
		DOM_Element Search(const DOM_Node &d, const DOMString &str);

		

	};

	string GetTime();

	// MAKE SURE LINKER LOADS GmeLib
	extern int DummyFL;
	static int *Dummy_For_Linker = &DummyFL;
}

//try & catch XML exception macros, which declare a block
//use careful, and together

#define TRY_XML_EXCEPTION try {
#define CATCH_XML_EXCEPTION(where)  }\
		catch(XMLException &e)\
		{\
			StrX msg = e.getMessage();\
			string what = "Udm: XMLException: ";\
			what += msg.localForm();\
			if (e.getType())\
			{\
				msg = e.getType();	\
				what += ", of type:";\
				what += msg.localForm();\
			}\
			what += ", in File: ";\
			what += e.getSrcFile();\
			what += ", in Line: ";\
			char * line_str = new char[20];/*should be enough for 32 bit integers */\
			if (line_str)\
			{\
				sprintf(line_str, "%d", e.getSrcLine());\
				what += line_str;\
				delete line_str;\
			}\
			what += where;\
			throw udm_exception(what);\
		}\


#endif//MOBIES_UDMDOM_H
