/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

#ifndef MOBIES_UDMGME_H
#define MOBIES_UDMGME_H

#ifndef MOBIES_UDMBASE_H
#include "UdmBase.h"
#endif

#include <comdef.h>


/*
CHANGELOG
	
	06/19/02	-	endre
	
			Added 
				
				  set<string> GmeDataNetwork::GetMetaRoleFilter(role):

			retrieves a set of strings containing all the possible
			GME CompositionRole values for a given CompositionChildRole

			Added 
			
				IMgaMetaRolePtr GmeObject::GetMetaRoleForKind(kind, mmodel)
			
			retrieves a MetaRole via an object of type 'kind' can be contained
			in a model of type 'mmodel'. Caller makes sure that there is a role 
			for such containment, and that role.target() is compatible with kind.
*/

namespace UdmGme
{
	typedef Udm::Object Object;

	void com_exception(HRESULT a, IUnknown *b, REFIID c);
	struct privdata;
	class GmeDataNetwork : public Udm::DataNetwork 
	{
		struct privdata &priv;
		Udm::BackendSemantics semantics;
		
		//map the Udm Composition Child Role to the corresponding set of MGA CompositionRole(MetaRole)s.
		typedef set<string> string_set;			//the compiler doesn't like when templates are nested
		map<Uml::CompositionChildRole, string_set> meta_role_filter_cache;


	public:
		void CountWriteOps();
		//type cache by name 
		map<string, const Uml::Class> meta_class_cache;
		
		GmeDataNetwork(const Udm::UdmDiagram &metainfo);
		~GmeDataNetwork();
		void CreateNew(const string &systemname, 
									const string &metalocator, const Uml::Class &rootclass, 
									enum Udm::BackendSemantics sem = Udm::CHANGES_PERSIST_ALWAYS);
		
		// For Udm-based Interpreters ///////////////////
		void OpenExisting(LPUNKNOWN pUnknown);
		Object Gme2Udm(LPUNKNOWN pUnknown);
		/////////////////////////////////////////////////

		void OpenExisting(const string &systemname, 
									const string &metalocator, 
									enum Udm::BackendSemantics sem = Udm::CHANGES_PERSIST_ALWAYS);
		void CloseWithUpdate();
		void CloseNoUpdate();
		void SaveAs(string systemname); 
		void CloseAs(string systemname); 

		void CommitEditSequence();
		void AbortEditSequence();
		Object ObjectById(Object::uniqueId_type id);

		set<string> GetMetaRoleFilter(Uml::CompositionChildRole role);

		static Udm::DataNetwork *factory(const Udm::UdmDiagram &meta) {
			return new GmeDataNetwork(meta);
		}
	private:
		int writeOps;
		bool hasOpened;
	};

	// MAKE SURE LINKER LOADS GmeLib
	extern int DummyFL;
	static int *Dummy_For_Linker = &DummyFL;
}

#endif//MOBIES_UDMGME_H
