/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

/*
CHANGELOG

  06/19/02	-	endre
		
		  Array attribute setters/getters override declaration

  03/19/02	-	endre
		
		  Added class StaticDataNetwork. See comments in .cpp
		  also moved to $/include/udm


  */

#ifndef MOBIES_UDMSTATIC_H
#define MOBIES_UDMSTATIC_H

#ifndef MOBIES_UDMBASE_H
#include "UdmBase.h"
#endif


#include <stdio.h> //file operations

namespace UdmStatic
{
	typedef Udm::Object Object;
	typedef Udm::ObjectImpl ObjectImpl;
	typedef ObjectImpl::uniqueId_type  uniqueId_type;

	class StaticObject;




	uniqueId_type GetIdForObject(StaticObject* p);
	StaticObject* GetObjectForId(uniqueId_type l);
	void InsertObject(StaticObject* so);
	

	//-------------------------StaticDataNetwork

	class StaticDataNetwork : public Udm::DataNetwork 
	{
		string systemname;
		Udm::BackendSemantics sem;
		
		//pointer to the datanetwork which can ObjectByID() metaobjects;
		//this is not always trivial, see comment in constructor
		Udm::DataNetwork * meta_dn;		
	public:
	
		static bool SafeObjectById;
		StaticDataNetwork(const Udm::UdmDiagram &metainfo);
		~StaticDataNetwork();
		Object ObjectById(Object::uniqueId_type id);

		
		virtual void CreateNew(const string &systemname, 
								const string &metalocator, const Uml::Class &rootclass, 
								enum Udm::BackendSemantics sem = Udm::CHANGES_PERSIST_ALWAYS);
		
		

		virtual void OpenExisting(const string &systemname, 
								const string &metalocator = "", 
								enum Udm::BackendSemantics sem = Udm::CHANGES_PERSIST_ALWAYS);

		
		virtual void SaveAs(string systemname);

		virtual void CloseWithUpdate()				{SaveAs(systemname);};
		virtual void CloseAs(string systemname)		{SaveAs(systemname);};		
		virtual void CommitEditSequence()			{SaveAs(systemname);};
		
		//do nothing here

		virtual void AbortEditSequence()			{};
		virtual void CloseNoUpdate()				{};
		
		bool isOpen()							
		{
			return rootobject;
		};

		static Udm::DataNetwork *factory(const Udm::UdmDiagram &meta) {
			return new StaticDataNetwork(meta);}
		//Static Data Network is _not_ a type safe data network
		// because m_type in StaticObject is an UML::Class reference!
		virtual bool IsTypeSafe() { return false;};
		unsigned long DeSerialize(FILE *f, map<unsigned long, const StaticObject*>& tr_map, Object& root_o);
		Udm::DataNetwork * GetMetaDn(){ return meta_dn;};

	};

	
	//------------------------StaticObject


	class StaticObject : public ObjectImpl
	{
	private:
		StaticDataNetwork * mydn;

		//archetype/derived/instances support
		set<StaticObject*> st_and_i;			// in this will be stored all derived and instantiated objects
		StaticObject* archetype;				// in this will be stored the archetype
		bool subtype;							// this tells whether is subtype or instance
												// if subtype, this variable is true
												// if instance, it's false


		bool real_archetype;					// true when *this is a real archetype(set by the user)
												// false when *this	is not a real archetype. Is a child
												// of an object which has a real  archetype, and thus all 
												// it's children are archetyped recursively.


		//we should store the attributes desynched from archetype
		
		
	public:

		StaticObject(const Uml::Class &meta, 
					int ref, 
					StaticObject * archetype = NULL, 
					const bool subtype = false,
					const bool real_archetype = true);
		void Destroy(bool root);		//destroy the object network
										// this deletes all the objects explicitly,
										// to be used only when the data network is going away ..


		
		static udm_exception e;
		~StaticObject();



	// --- reference counting
		int refCount;
	//desynched attributes
		set<uniqueId_type> desynched;

		virtual ObjectImpl *clone();
		virtual void release();
		virtual Udm::DataNetwork *__getdn();
		const Uml::Class &m_type;
	
		virtual const Uml::Class &type() const;
		//void dumpref() {cout << refCount << endl;};
		//for debugging only!
		int ref(){  return refCount;};
	
	// --- order
	
		virtual uniqueId_type uniqueId() const;

	// --- attributes (single-value /no-array/ type)
		typedef map<uniqueId_type, string> stringAttrs_type;
		stringAttrs_type stringAttrs;

		typedef map<uniqueId_type, bool> booleanAttrs_type;
		booleanAttrs_type booleanAttrs;

	//	typedef map<uniqueId_type, long> integerAttrs_type;
	//	integerAttrs_type integerAttrs;

		typedef map<uniqueId_type, __int64> longintAttrs_type;
		longintAttrs_type longintAttrs;

		typedef map<uniqueId_type, double> realAttrs_type;
		realAttrs_type realAttrs;
	
	// --- attributes (multiple value /array/ type)
		typedef vector<string> string_array;
		typedef map<uniqueId_type, string_array> stringAttrArrs_type;
		stringAttrArrs_type stringAttrArrs;
		
		typedef vector<bool> bool_array;
		typedef map<uniqueId_type, bool_array> booleanAttrArrs_type;
		booleanAttrArrs_type booleanAttrArrs;

		//typedef vector<long> long_array;
		//typedef map<uniqueId_type, long_array> integerAttrArrs_type;
		//integerAttrArrs_type integerAttrArrs;

		typedef vector<__int64> longint_array;
		typedef map<uniqueId_type, longint_array> longintAttrArrs_type;
		longintAttrArrs_type longintAttrArrs;


		typedef vector<double> double_array;
		typedef map<uniqueId_type, double_array> realAttrArrs_type;
		realAttrArrs_type realAttrArrs;
		


	// --- attribute setters/getters  for single value (no-array) type attributes
		virtual string getStringAttr(const Uml::Attribute &meta) const;
		virtual void setStringAttr(const Uml::Attribute &meta, const string &a, const bool direct = true);
		
		virtual bool getBooleanAttr(const Uml::Attribute &meta) const;
		virtual void setBooleanAttr(const Uml::Attribute &meta, bool a, const bool direct = true);
		
		virtual __int64 getIntegerAttr(const Uml::Attribute &meta) const;
		virtual void setIntegerAttr(const Uml::Attribute &meta, __int64 a, const bool direct = true);
		
		virtual double getRealAttr(const Uml::Attribute &meta) const;
		virtual void setRealAttr(const Uml::Attribute &meta, double a, const bool direct = true);
	
	//	--- attribute setters/getters for multiple value (array) type attributes

		virtual vector<string> getStringAttrArr(const Uml::Attribute &meta) const;
		virtual void setStringAttrArr(const Uml::Attribute &meta, const vector<string> &a, const bool direct = true);

		virtual vector<bool> getBooleanAttrArr(const Uml::Attribute &meta) const;
		virtual void setBooleanAttrArr(const Uml::Attribute &meta, const vector<bool> &a, const bool direct = true);

		virtual vector<__int64> getIntegerAttrArr(const Uml::Attribute &meta) const;
		virtual void setIntegerAttrArr(const Uml::Attribute &meta, const vector<__int64> &a, const bool direct = true);

		virtual vector<double> getRealAttrArr(const Uml::Attribute &meta) const;
		virtual void setRealAttrArr(const Uml::Attribute &meta, const vector<double> &a, const bool direct = true);



	// --- containment, maintance(create, delete)
		StaticObject *m_parent;	// not reference counted

		typedef multimap<uniqueId_type, StaticObject*> children_type;
		children_type m_children;	// reference counted

		virtual ObjectImpl *getParent(const Uml::CompositionParentRole &role) const;
		virtual void setParent(ObjectImpl *a, const Uml::CompositionParentRole &role, const bool direct = true);
		virtual void detach();
		virtual vector<ObjectImpl*> getChildren(const Uml::CompositionChildRole &meta, const Uml::Class &cls) const;
		virtual void setChildren(const Uml::CompositionChildRole &meta, const vector<ObjectImpl*> &a, const bool direct = true);
		virtual ObjectImpl *createChild(const Uml::CompositionChildRole &childrole, const Uml::Class &meta, const Udm::ObjectImpl* archetype = NULL, const bool subtype = false, const bool real_archetype = true);

	// --- associations
		typedef multimap<uniqueId_type, StaticObject*> assoc_type;
		assoc_type associations;	// not reference counted, bidirectional

		virtual vector<ObjectImpl*> getAssociation(const Uml::AssociationRole &meta, int mode = Udm::TARGETFROMPEER) const;
		virtual void setAssociation(
			const Uml::AssociationRole &meta, 
			const vector<ObjectImpl*> &nvect, 
			int mode = Udm::TARGETFROMPEER,
			const bool direct = true);
	// --- persistency
		unsigned long Serialize(FILE * f);											// return the length of the object
		void ValidatePointers(map<unsigned long, const StaticObject *>& tr_map);	// changing the associations and the children to the new pointers

	// --- acrhetype/derived/instances
		virtual vector<ObjectImpl*> getDerived() const;
		virtual vector<ObjectImpl*> getInstances() const;
		virtual ObjectImpl* getArchetype() const;
		virtual bool hasRealArchetype() const;

		
		StaticObject * FindCorrespondingObjectInStOrITree( StaticObject * where);
		StaticObject * FindCorrespondingObjectInStOrI( StaticObject * where);
		
	


	};



	Object CreateObject(const Uml::Class &meta);
	void CreateComposition(const Object &parent, const Uml::CompositionChildRole &childRole,
		const Object &child, const Uml::CompositionParentRole &parentRole);
	void CreateAssociation(const Object &src, const Uml::AssociationRole &dstRole,
		const Object &dst, const Uml::AssociationRole &srcRole);
}


#endif//MOBIES_UDMSTATIC_H
