// ClipRegion.cpp: implementation of the CClipRegion class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TaskPlot.h"
#include "ClipRegion.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// To draw a line correctly, it is necessary to clip it on the client
// area. This insure that the GDI coordinates are in the range of
// -32768 <-> 32767.

CClipRegion::CClipRegion()
{
	memset(ClipRect, 0, MaxReg * sizeof(long));
}

CClipRegion::CClipRegion(CRect &rectClip)
{
	ClipRect[right]  = rectClip.right;
	ClipRect[left]   = rectClip.left;
	ClipRect[top]    = rectClip.top;
	ClipRect[bottom] = rectClip.bottom;
}

int CClipRegion::ClipRegion(long* p)
{
	int Reg = 0;
	if		(*p >= ClipRect[right])  Reg |= RegRight;
	else if (*p <= ClipRect[left])   Reg |= RegLeft;
	p++;
	if		(*p >= ClipRect[top])    Reg |= RegTop;
	else if (*p <= ClipRect[bottom]) Reg |= RegBottom;
	return Reg;
}

void CClipRegion::Intersect(long *r, long *a, long *b, short Plane)
{
	static short c[MaxReg][2] = { x,y, x,y, y,x, y,x };
	short c1 = c[Plane][x];
	short c2 = c[Plane][y];
	long mul, div, r1, r2;

	mul = ClipRect[Plane] - a[c1];
	div = b[c1] - a[c1];
	r1 = a[c1] + MulDiv(b[c1] - a[c1], mul, div);
	r2 = a[c2] + MulDiv(b[c2] - a[c2], mul, div);
	r[c1] = r1; r[c2] = r2;
}

BOOL CClipRegion::PtInRect(CPoint &p)
{
	if (p.x >= ClipRect[right])  return FALSE;
	if (p.x <  ClipRect[left])   return FALSE;
	if (p.y >  ClipRect[top])    return FALSE;
	if (p.y <= ClipRect[bottom]) return FALSE;
	return TRUE;
}

BOOL CClipRegion::ClipLine(CPoint &p1, CPoint &p2)
{
	short Reg1, Reg2;
	register short i;
	static short ClipReg[MaxReg] =	{ RegRight, RegLeft, RegTop, RegBottom };

	long *p = &p1.x, *q = &p2.x;

	Reg1 = ClipRegion(p);
	Reg2 = ClipRegion(q);

	if (Reg1 | Reg2)
	{
		for (i = 0; i < MaxReg; i++)
			if ((Reg1 == Reg2) && (Reg2 == ClipReg[i])) return FALSE;

		for (i = MaxReg - 1; i >= 0; i--)
		{
			if (Reg1 & Reg2 & ClipReg[i]) return FALSE;
			if ((Reg1 | Reg2) & ClipReg[i])
			{
				if (Reg1 & ClipReg[i])
					Intersect(p, q, p, i);
				else
					Intersect(q, p, q, i);
				if (i >= 0)
				{
					Reg1 = ClipRegion(p);
					Reg2 = ClipRegion(q);
				}
			}
		}
	}
	return TRUE;
}

BOOL CClipRegion::ClipRectangle(CRect &rect)
{
	CPoint p1, p2;

	if(IsRectOnRect(rect))
		return TRUE;
	
	HRGN rgn = CreateRectRgn(rect.left,rect.top,rect.right,rect.bottom);
	HRGN rgn_clip = CreateRectRgn(ClipRect[left],ClipRect[top],ClipRect[right],ClipRect[bottom]);
	HRGN out = CreateRectRgn(0,0,10,10);

	int ret = CombineRgn(out,rgn,rgn_clip,RGN_AND);
	if(ret == SIMPLEREGION)
	{
		GetRgnBox(out, &rect);
		return TRUE;
	}
	return FALSE;
}

BOOL CClipRegion::IsRectOnRect(CRect& rect)
{
	if (rect.left   >= ClipRect[right])  return FALSE;
	if (rect.right  <= ClipRect[left])   return FALSE;
	if (rect.top    <= ClipRect[bottom]) return FALSE;
	if (rect.bottom >= ClipRect[top])    return FALSE;
	return TRUE;
}
