#include "stdafx.h"
#include "RealTimePlot.h"

#define ZOOM_VIEW 0

typedef struct 
{
	int type;
	COLORREF color;
}MarkSetTag;

MarkSetTag MarkSet[MAX_EVENT_NUMBER];

double LCMsub(double i, double j, double k)
{
	if ((DWORD)j % (DWORD)i == 0)
		return j;
	else
		return LCMsub(i, j+k, k);
}

double CalculateLCM(double i, double j)
{
	if ((DWORD)j % (DWORD)i == 0)
		return j;
	else
		return LCMsub(i, 2*j, j);
}

double CalculateLCM(int *array, int size)
{
	int n = size;
	if(size < 2) return array[0];
	double tem,old;
	
	tem = array[0];
	old = tem;
	for(int i=1; i<n; i++)
	{
		tem = CalculateLCM(tem, array[i]);
		
		// limitation of calculation
		if(tem > 500)
			return old;
		old = tem;
	}
	return tem;
}

/*
double CalculateLCM(int *array, int count)
{
    double X = array[0];
    int i = 0;

    while ( true ) 
	{
        if ( ((DWORD)X % (DWORD)array[i]) == 0 ) 
		{
            if ( i >= count-1 )
                break;
            i++;
        }

        else 
		{
            X = X + 1;
            i = 0;
        }
    }

    return X;
}
*/
int CalculateGCD(int a, int b) 
{
	int c = a, d = b, e;
	while (d) 
	{ 
		e = c % d; c = d; d = e; 
	}
	return c;
}

int issame(double x,double z)
{                
	if(fabs(x-z) <= 1.0e-5) return 1;	

	return 0;             
}

int lequal(double x,double z)
{
	if(issame(x,z) || x < z)	return 1;
	else						return 0;	
}

int bequal(double x,double z)
{
	if(issame(x,z) || x > z)	return 1;
	else						return 0;	
}

char* fnum2str(double V)
{
	static char numstr[50];
	int len = sprintf( numstr,"%-6.4lf",V);
	for( int i = len-1; i >= 0 ; i-- )
	{
		if( numstr[i] == '0') numstr[i] = '\x0';
		else { 
			if( numstr[i] == '.') {
				numstr[i] = '\x0';
			}
			break;
		}
	}
	return numstr;
}

//////////////////////////////////////////////////////////////////////////////
CRealTimePlot::CRealTimePlot()
{
	m_bSetingFailed	= true;
	m_sHigh			= 17;
	m_nMaxTime		= 0;
	m_nCursorTime	= 0;
	m_pCurLine		= NULL;
	m_wdTickLine	= 2;
	m_crTickLine	= RGB(255, 0, 0);
	m_nDataPerLine	= 400;

	m_sizeDoc = CSize(2960, 2100);	// default to A4 landscape, MM_LOMETRIC
	m_x0 = 0;//m_sizeDoc.cx / 2;
	m_y0 = 0;//m_sizeDoc.cy / 2 ;
	m_Scale = ZOOM_ONE;

	MarkSet[RELAESE_EVENT].type = TRIANGLE;//FTRIANGLE;
	MarkSet[RELAESE_EVENT].color = RED;

	MarkSet[START_EVENT].type = TRIANGLE;//FTRIANGLE;
	MarkSet[START_EVENT].color = BLACK;

	MarkSet[PREEMPTED_EVENT].type = TRIANGLE;//FTRIANGLE;
	MarkSet[PREEMPTED_EVENT].color = LTBLUE;

	MarkSet[RESUME_EVENT].type = TRIANGLE;//FTRIANGLE;
	MarkSet[RESUME_EVENT].color = YELLOW;
	
	MarkSet[COMPLETE_EVENT].type = REV_TRIANGLE;//REV_FTRIANGLE;
	MarkSet[COMPLETE_EVENT].color = LTBLUE;

	m_curproc = 0;
}

CRealTimePlot::~CRealTimePlot()
{
}

RECT CRealTimePlot::TickBarSize()
{
	m_SRect		   = m_PlotRect;
	m_SRect.top	   = m_SRect.top - m_sHigh;
	m_SRect.bottom = m_SRect.top  + m_sHigh - 1;
	m_SRect.left   = m_SRect.left - 7;
	m_SRect.right  = m_SRect.right + 7;

	return m_SRect;
}

bool CRealTimePlot::SetDataScale(int pindex, double xmin, double ymin, double xmax, double ymax)
{
	if (ymax <= ymin)
		return false; 

	m_DataScale[pindex].xmin	= xmin;
    m_DataScale[pindex].ymin	= ymin;
    m_DataScale[pindex].xmax	= xmax;
    m_DataScale[pindex].ymax	= ymax;
	return true; 
}
/*
SetDataScale should be called before calling SetScreenScale
*/
bool CRealTimePlot::InitComplete(int pindex, double maxtimetoshow)
{
	double LCM = 0.0;
	int iTask,jTask;
	int nTasks = GetMaxChannel(pindex);
	
	SetYNumOfGridTicks(nTasks);
	CTaskChannel *aTaskLine,*aSubTask;
	int *parray = new int[nTasks];

	if(nTasks < 1)
		return false;

	for(iTask=0; iTask< nTasks; iTask++)
	{
		aTaskLine = m_procs.Get(pindex, iTask);
		parray[iTask] = (int)aTaskLine->m_period;
	}

	// find min period
	double min_period = 10000000;
	for(iTask=0; iTask< nTasks; iTask++)
	{
		aTaskLine = m_procs.Get(pindex, iTask);
		if(min_period > aTaskLine->m_period)
			min_period = aTaskLine->m_period;
	}

	// find max period
	double max_period = 0;
	for(iTask=0; iTask< nTasks; iTask++)
	{
		aTaskLine = m_procs.Get(pindex, iTask);
		if(max_period < aTaskLine->m_period)
			max_period = aTaskLine->m_period;
	}

/*	for(iTask=0; iTask< nTasks; iTask++)
	{
		if(parray[iTask] == 0)
		{
			LCM = max_period;
			break;
		}
	}
*/
	LCM = max_period;
	double ratio = max_period / min_period;
	if(ratio > 50.0)
		maxtimetoshow = min_period * 50;

	//************************************************
	if(LCM < 1.0E-5)
	{
		if(maxtimetoshow > 1.0E-5)
			LCM = maxtimetoshow;
		else
			LCM = CalculateLCM(parray, nTasks);
	}

	//limit display size
	double max_display = max_period*2;
	if(LCM > max_display)
	{
		LCM = max_display;
	}
	//************************************************

	SetDataScale(pindex, 0, 0, LCM, nTasks * 30);

	for(iTask=0; iTask< nTasks; iTask++)
	{
		aTaskLine = m_procs.Get(pindex, iTask);

		aTaskLine->m_max_time = LCM;
		aTaskLine->SetPeriodicEvents();
	}

	for(iTask=0; iTask < nTasks; iTask++)
	{
		CString temp;
		aTaskLine = m_procs.Get(pindex, iTask);
		
		aTaskLine->SortTimeLine();
	}
	OutputDebugString("SetPeriodicEvents() is done.\r\n");

	// for preemption
	for(iTask=0; iTask < nTasks - 1; iTask++)
	{
		aTaskLine = m_procs.Get(pindex, iTask);

		for(jTask=iTask+1; jTask < nTasks; jTask++)
		{
			aSubTask = m_procs.Get(pindex, jTask);

			aSubTask->SetPreemptionEvents(aTaskLine);
		}
	}
	for(iTask=0; iTask < nTasks; iTask++)
	{
		CString temp;
		aTaskLine = m_procs.Get(pindex, iTask);

		aTaskLine->SortTimeLine();
	}
	OutputDebugString("SetPreemptionEvents() is done.\r\n");

	// for completion
	for(iTask=0; iTask < nTasks; iTask++)
	{
		aTaskLine = m_procs.Get(pindex, iTask);

		aTaskLine->SetCompleteEvent();
	}
	for(iTask=0; iTask < nTasks; iTask++)
	{
		CString temp;
		aTaskLine = m_procs.Get(pindex, iTask);
		aTaskLine->SortTimeLine();
	}
	OutputDebugString("SetCompleteEvent() is done. \r\n");

	// sort the events as timed.
	CString out;
	out.Format("RELAESE=1,START=2,COMPLETE=3,PREEMPTED=4,BLOCK=5,RESUME=6,DEADLINE=9,\r\n");
	for(iTask=0; iTask < nTasks; iTask++)
	{
		CString temp;
		aTaskLine = m_procs.Get(pindex, iTask);
		aTaskLine->SortTimeLine();

#ifdef _DEBUG
		aTaskLine->PrintTimeLine(temp);
		out += temp;
#endif
	}

	// generate debugging file
#ifdef _DEBUG
	int debug = 0;
	if(debug)
	{
		FILE *fp;
		fp = fopen("c:\\debug.log","w");
		if(fp != NULL) 
		{
			fwrite ((LPCSTR)out, lstrlen ((LPCSTR)out),1,fp);
			fwrite ("\015\012", 2,1,fp);  
			fclose (fp);
		}
	}
#endif

	m_DataScale[pindex].xmin = 0;
	m_DataScale[pindex].xmax = LCM;

	assert(m_DataScale[pindex].xmin >= 0);
	assert(m_DataScale[pindex].xmax >= 0);
	assert(m_DataScale[pindex].xmax - m_DataScale[pindex].xmin > 0);

	EnableLegend(true);
	m_bLegendShadow = false;

	XAxisTitle(pindex, "time");

	delete parray;
	return	true;
}

void CRealTimePlot::Legend(COLORREF cr, int taskIndex,int rowindex, const char* Name)
{
	int pindex = m_curproc;
	if (m_Font.m_hObject == NULL) return;

	int y1,y2;
	y1 = (int)(GT[pindex] - 1.0 * PY[pindex] * (taskIndex)/ YGridTicks);
	y2 = (int)(GT[pindex] - 1.0 * PY[pindex] * (taskIndex+1) / YGridTicks);
	int yb = (y1+y2)/2 + m_fontHeight/2;

	int n  = (m_Rect.right - GR[pindex]) / 20 + 1;
	int xb = GR[pindex] + n;
	int xe = xb + n;

	int y  = yb - (rowindex-1) * (m_fontHeight*1.2) ;

//	DrawLine(xb, y, xe, y);
	
	int bm  = ::SetBkMode(m_DC->m_hDC, TRANSPARENT);
	HFONT hOldFont = (HFONT)::SelectObject(m_DC->m_hDC, m_Font);
	::SetTextColor(m_DC->m_hDC, cr); 
	SetStringAlign(LEFT, CENTER);

	DrawClipText(xe, y, CString(Name));
	
	::SelectObject(m_DC->m_hDC, hOldFont);
	::SetBkMode(m_DC->m_hDC, bm);
}

void CRealTimePlot::ChannelTitle(const char* Title, int index)
{
	int Pos = LEFT;
	CString name = Title;

	int pindex = m_curproc;

	if (m_Font.m_hObject == NULL) return;

	int bm  = ::SetBkMode(m_DC->m_hDC, TRANSPARENT);
	HFONT hOldFont = (HFONT)::SelectObject(m_DC->m_hDC, m_Font);
	::SetTextColor(m_DC->m_hDC, m_nTitleColor); 
	SetStringAlign(CENTER, CENTER);

	int y1,y2;
	y1 = (int)(GT[pindex] - 1.0 * PY[pindex] * (index) / YGridTicks);
	y2 = (int)(GT[pindex] - 1.0 * PY[pindex] * (index+1) / YGridTicks);

	if (Pos == LEFT)
	{
		DrawClipText(m_Rect.left + (GL[pindex] - m_Rect.left)/10, (y2 + y1)/2+m_fontHeight/2, name, m_PlotRect.left - (GL[pindex] - m_Rect.left) / 10);
	}
	else
	{
		DrawClipText(m_Rect.right - (m_Rect.right - GR[pindex]) / 2, (y2 + y1)/2, name);
	}

	::SelectObject(m_DC->m_hDC, hOldFont);
	::SetBkMode(m_DC->m_hDC, bm);
}

void CRealTimePlot::Show()
{
	int pindex = m_curproc;
	int nTasks = GetMaxChannel(pindex);
	static int epre[MAXTASK];
	static double tpre[MAXTASK];
	static double xpre[MAXTASK], ypre[MAXTASK];
	static double preemt_x[MAXTASK],preemt_y[MAXTASK];
	static BOOL isPreempted[MAXTASK], isCompleted[MAXTASK],isStarted[MAXTASK];

	if(nTasks < 1)
		return ;

	SetYNumOfGridTicks(nTasks);

	for(int iTask=0; iTask< nTasks; iTask++)
	{
		CTaskChannel *aTaskLine = m_procs.Get(pindex,iTask);

		ChannelTitle(aTaskLine->m_sName, iTask);

		int nPoint = aTaskLine->GetSize();
		int event,mode;
		double x,y;
		int barht,marksize;
		COLORREF	color;
		double time;
		isPreempted[iTask] = 0;
		isCompleted[iTask] = 0;
		isStarted[iTask] = 0;
		tpre[iTask] = 0;
		epre[iTask] = -1;

		for(int i=0; i< nPoint; i++)
		{
			event = aTaskLine->m_pValues[i].event;
			time = aTaskLine->m_pValues[i].time;

			if(time > m_DataScale[pindex].xmax) break;

			mode = MarkSet[event].type;
			color = MarkSet[event].color;

			barht = (int)(PY[pindex] / nTasks * 0.5);
			x = CalXAxis(pindex, time);

			marksize = barht / 10;
			if(marksize < 6) marksize = 6;

			switch(event)
			{
			case RELAESE_EVENT:

				y = (int)CalYAxis(pindex, iTask+1);

				DrawClipMarker((int)x,(int)y, mode,marksize, color);

				if(isCompleted[iTask] !=  1)
				if(isPreempted[iTask] < 1)
				if(time > 0)
				{
					//color = PURPLE;
					color = LTBLUE;
					FillClipRectangle((int)xpre[iTask], (int)ypre[iTask], (int)x,(int)y + barht, color);
				}

				xpre[iTask] = x;
				ypre[iTask] = y;
				isCompleted[iTask] = 0;
				isStarted[iTask] = 0;
				break;

			case START_EVENT:
				y = (int)CalYAxis(pindex, iTask+1) + barht;

				DrawRectangle((int)xpre[iTask], (int)ypre[iTask], (int)x, (int)y, color);

				xpre[iTask] = x;
				ypre[iTask] = y - barht;

				isStarted[iTask] = 1;
				break;

			case COMPLETE_EVENT:

				y = (int)CalYAxis(pindex, iTask+1) + barht;
				
				FillClipRectangle((int)xpre[iTask], (int)ypre[iTask], (int)x,(int)y, color);

				// completion mark
				DrawClipMarker((int)x,(int)y,mode,marksize, color);

				xpre[iTask] = x;
				ypre[iTask] = y - barht;

				isCompleted[iTask] = 1;
				break;

			case PREEMPTED_EVENT:

				y = (int)CalYAxis(pindex, iTask+1) + barht;
				
				if(isStarted[iTask] == 1)
				if(isCompleted[iTask] <  1)
				{
					FillClipRectangle((int)xpre[iTask], (int)ypre[iTask], (int)x,(int)y, color);
				}

				if(isPreempted[iTask] == 0)
				{
					preemt_x[iTask] = x;
					preemt_y[iTask] = y - barht;
				}
				
				isPreempted[iTask] = isPreempted[iTask] + 1;
				break;

			case RESUME_EVENT:

				y = (int)CalYAxis(pindex, iTask+1) + barht;

				FillClipRectangle((int)preemt_x[iTask], (int)preemt_y[iTask], (int)x,(int)y, color);
				
				isPreempted[iTask] = isPreempted[iTask] - 1;
				xpre[iTask] = x;
				ypre[iTask] = y - barht;
				break;

			case DEADLINE_EVENT:
				{
				y = (int)CalYAxis(pindex, iTask+1);
				
				if(isCompleted[iTask] > 0)
				{
					barht = (int)(PY[pindex] / nTasks * 0.7);
					color = GREEN;
					DrawClipLine((int)x,(int)y, (int)x,(int)y + barht, color);
//					DrawClipLine((int)x+1,(int)y, (int)x+1,(int)y - barht, color);
				}
				else if(aTaskLine->ExistEvent(time, COMPLETE_EVENT) == 0)
				{ // MISS DEADLINE
					barht = (int)(PY[pindex] / nTasks * 0.7);
					color = RED;
					DrawClipLine((int)x,(int)y, (int)x,(int)y + barht, color);
					DrawClipLine((int)x+1,(int)y, (int)x+1,(int)y + barht, color);

					barht = (int)(PY[pindex] / nTasks * 0.7);
					mode = FDIAMOND;
					color = RED;
					DrawClipMarker((int)x,(int)y + barht,mode,marksize, color);
				}
				}
				break;

			case BLOCK_EVENT:
				break;
			
			case LOCK_EVENT:
				break;
			case UNLOCK_EVENT:
				break;
			default:
				AfxMessageBox("Something Wrong");
				break;
			}

			if(tpre[iTask] != time)
			{
				tpre[iTask] = time;
				epre[iTask] = event;
			}
		}
	}
}

/*
Higher priority task should be called first.
*/
int CRealTimePlot::AddChannel(int pindex, const char* name, int priority, double period, double deadline,double wcet, double offset)
{
	CTaskChannel *aTask = m_procs.AddChannel(pindex);

//	aTask->m_nColor			= color;
	aTask->m_sName			= name;
	aTask->m_period			= period;
	aTask->m_deadline		= deadline;
	aTask->m_priority		= priority;
	aTask->m_wcet			= wcet;
	aTask->m_offset			= offset;
	
	if(!aTask->Reset(m_nDataPerLine))
	{
		//aProc->RemoveAt(n);
		return -1;
	}

	return m_procs.GetSize(pindex);
}

/*
bool CRealTimePlot::RemoveALine(int pindex, int Index)
{
	int n = GetMaxChannel(pindex);
	if (n <= 0 || n <= Index)
		return false;

	m_LineArray[pindex].RemoveAt(Index);
	return true;
}
*/
void CRealTimePlot::RemoveAll(int pindex)
{
	int n = GetMaxChannel(pindex);

	for(int i = n-1; i >= 0; i --)
		m_procs.Remove(pindex,i);
}

void CRealTimePlot::DrawXAxisTimeTicks(int x, int y, double& ticks)
{
//	sprintf(m_sTempStr, "%d:%d:%d", ticks/3600, (ticks/60)%60, ticks%60);
//	PrintString(x, y, 0, m_sTempStr);	
}

void CRealTimePlot::XAxis(int pindex)
{
	int	  xb, yb, xe, ye;   
    int   i, j;                                  
	char  str[32];
	float value;

	yb = GB[pindex];
	j = XTicks / 10;
	for(i = 0; i <= XTicks; i ++) 
	{
		xb = xe = (int)(GL[pindex] + 1.0 * PX[pindex] * i / XTicks );
		if((i % j) == 0)
		{
			ye = GB[pindex] - m_bM / 7;
			value = (float)(m_DataScale[pindex].xmin + i * (m_DataScale[pindex].xmax - m_DataScale[pindex].xmin) / XTicks);
			lstrcpy(str,fnum2str(value));
			//PrintString(xb, GB - m_bM / 5, 0, str);
			DrawClipText(xb, GB[pindex] - m_bM / 5, CString(str));
		}
		else 
		{
			ye = GB[pindex] - m_bM / 14;
		}

		if (i > 0 && i < XTicks)
		{
			DrawClipLine(xb, yb, xe, ye);
		}
	}
}

double CRealTimePlot::CalXAxis(int pindex, double time)
{
	double ret = (GL[pindex] + (time - m_DataScale[pindex].xmin) / m_DataScale[pindex].dx);
	return ret;
}

int CRealTimePlot::CalYAxis(int pindex, int tindex)
{
	int m_nTasksToShow = GetMaxChannel(pindex);
	
	if(m_nTasksToShow < 1)	return 0;
	int ret = (int)(GT[pindex] - 1.0 * PY[pindex] * tindex / m_nTasksToShow );
	return ret;
}

void CRealTimePlot::YAxis(int pindex)
{
	int m_nTasksToShow = GetMaxChannel(pindex);
	if(m_nTasksToShow < 1)	return;

	for(int i = 0; i <= m_nTasksToShow; i ++) 
	{
		yb = ye = CalYAxis(pindex, i);
		xe = GR[pindex];
		xb = GL[pindex];

		DrawClipLine(xb, yb, xe, ye);
	}

//	PrintTime();
}

void CRealTimePlot::Grid(int pindex)
{
	HPEN hPen	= ::CreatePen(PS_DOT, 0, m_nGridColor);
	HPEN hOldPen = (HPEN)::SelectObject(m_DC->m_hDC, hPen);

	int xb,xe;
	//int yb,ye;
	int j = XTicks / 10;

	for(int i = 0; i < XTicks; i ++) 
	{
		xb = xe = (int)(GL[pindex] + 1.0 * PX[pindex] * i / XTicks );
		if((i % j) == 0)
		{
			DrawClipLine(xb, GT[pindex] + 1, xe, GB[pindex] - 1);
		}
	}

	for(i = 1; i < YGridTicks; i ++) 
	{
//		DrawClipLine(GL + 1, yb, GR - 1, ye);
	}

	::SelectObject(m_DC->m_hDC, hOldPen);
	::DeleteObject(hPen);
}

void CRealTimePlot::DrawLegend()
{
	int pindex = m_curproc;
	int n = GetMaxChannel(pindex);
	if(n < 1)	return;

	if (m_bLegendShadow)
		DrawShadow(n);

	for(int i = 0; i < n; i ++)
	{
		m_pCurLine = m_procs.Get(pindex, i);

		if (m_bEnableLegend)
		{
			m_CurPen = ::CreatePen(m_pCurLine->m_nLineStyle, m_pCurLine->m_nLineWidth, m_pCurLine->m_nColor);
			m_OldPen = (HPEN)::SelectObject(m_DC->m_hDC, m_CurPen);

			int rowindex = 1;
			CString str,str1;
			str.Format("Period=%s",fnum2str(m_pCurLine->m_period));
			Legend(m_pCurLine->m_nColor, i, rowindex++, (LPCTSTR)str);

//			str.Format("Priority=%d",m_pCurLine->m_priority);
//			Legend(m_pCurLine->m_nColor, i, rowindex++, (LPCTSTR)str);

//			str.Format("Deadline=%s",fnum2str(m_pCurLine->m_deadline));
//			Legend(m_pCurLine->m_nColor, i, rowindex++, (LPCTSTR)str);

//			str.Format("WCET=%s",fnum2str(m_pCurLine->m_wcet));
//			Legend(m_pCurLine->m_nColor, i, rowindex++, (LPCTSTR)str);

//			str.Format("Start=%s",fnum2str(m_pCurLine->m_offset));
//			Legend(m_pCurLine->m_nColor, i, rowindex++, (LPCTSTR)str);

			::SelectObject(m_DC->m_hDC, m_OldPen);
			::DeleteObject(m_CurPen);
		}
	}
}

void CRealTimePlot::DrawCurrentLine()
{
	m_CurPen = ::CreatePen(m_pCurLine->m_nLineStyle, m_pCurLine->m_nLineWidth, m_pCurLine->m_nColor);
	m_OldPen = (HPEN)::SelectObject(m_DC->m_hDC, m_CurPen);

	::SelectObject(m_DC->m_hDC, m_OldPen);
	::DeleteObject(m_CurPen);
}

void CRealTimePlot::Redraw(HWND hWnd)
{
	RECT rt;
	int pindex = m_curproc;

	rt.left   = GL[pindex];
	rt.top    = GT[pindex] - 1;
	rt.right  = GR[pindex];
	rt.bottom = m_Rect.bottom - 1;
	::InvalidateRect(hWnd, &rt, FALSE);
}

void CRealTimePlot::SetPrintTime(double time, int flag)
{
	if (flag == MaxTime)
		m_nMaxTime = time / 1000;
	else
		m_nCursorTime = time / 1000;
}

void CRealTimePlot::PrintTime()
{
	int n = 5;
/*	if (m_bPrinting)	n *= m_nPrintScale;
	n = m_Rect.bottom - n - 1;
	SetStringAlign(LEFT, BOTTOM);
	ShowTime(GL, n, "Cursor time", m_nCursorTime);
	SetStringAlign(RIGHT, BOTTOM);
	ShowTime(GR, n, "Total lapse time", m_nMaxTime);
*/
}

void CRealTimePlot::ShowTime(int x, int y, const char* Tag, double& cTime)
{
//	sprintf(m_sTempStr, "%s (%d:%d:%d)", Tag, cTime / 3600, (cTime / 60) % 60, cTime % 60);
//	PrintString(x, y, 0, m_sTempStr);	
}

void CRealTimePlot::DrawClipLine(long x1, long y1, long x2, long y2, COLORREF color)
{
	HPEN hCurPen;
	HPEN hOldPen;
		
	if(color != -1)
	{
		hCurPen = ::CreatePen(PS_SOLID, 0, color);
		hOldPen = (HPEN)m_DC->SelectObject(hCurPen);
	}

	CPoint p1(xPos(x1), yPos(y1));
	CPoint p2(xPos(x2), yPos(y2));
	if (m_clipReg.ClipLine(p1, p2))
	{
		m_DC->MoveTo(p1.x, p1.y);
		m_DC->LineTo(p2.x, p2.y);
	}

	if(color != -1)
	{
		m_DC->SelectObject(hOldPen);
		::DeleteObject(hCurPen);
	}	
}

void CRealTimePlot::DrawClipRect(const CRect &rect, COLORREF color)
{
	HPEN hCurPen;
	HPEN hOldPen;
		
	if(color != -1)
	{
		hCurPen = ::CreatePen(PS_SOLID, 0, color);
		hOldPen = (HPEN)m_DC->SelectObject(hCurPen);
	}
	CPoint p1, p2;

	p1 = CPoint(xPos(rect.left), yPos(rect.top));
	p2 = CPoint(xPos(rect.right), yPos(rect.top));
	if (m_clipReg.ClipLine(p1, p2))
	{
		m_DC->MoveTo(p1.x, p1.y);
		m_DC->LineTo(p2.x, p2.y);
	}

	p1 = CPoint(xPos(rect.right), yPos(rect.top));
	p2 = CPoint(xPos(rect.right), yPos(rect.bottom));
	if (m_clipReg.ClipLine(p1, p2))
	{
		m_DC->MoveTo(p1.x, p1.y);
		m_DC->LineTo(p2.x, p2.y);
	}

	p1 = CPoint(xPos(rect.right), yPos(rect.bottom));
	p2 = CPoint(xPos(rect.left), yPos(rect.bottom));
	if (m_clipReg.ClipLine(p1, p2))
	{
		m_DC->MoveTo(p1.x, p1.y);
		m_DC->LineTo(p2.x, p2.y);
	}

	p1 = CPoint(xPos(rect.left), yPos(rect.bottom));
	p2 = CPoint(xPos(rect.left), yPos(rect.top));
	if (m_clipReg.ClipLine(p1, p2))
	{
		m_DC->MoveTo(p1.x, p1.y);
		m_DC->LineTo(p2.x, p2.y);
	}

	if(color != -1)
	{
		m_DC->SelectObject(hOldPen);
		::DeleteObject(hCurPen);
	}	
}

void CRealTimePlot::FillClipRectangle(int x1, int y1, int x2,int y2, COLORREF color)
{
	HBRUSH hBrush;
	HBRUSH hBrold;

	if(color != -1)
	{
		hBrush = ::CreateSolidBrush(color);
		hBrold = (HBRUSH)::SelectObject(m_DC->m_hDC, hBrush);
	}

	CRect rc;
	rc.left = xPos(x1);
	rc.top = yPos(y1);
	rc.right = xPos(x2);
	rc.bottom = yPos(y2);

	if (-32768 <= rc.left && rc.left <= 32767)
//		if (m_clipReg.IsRectOnRect(rc))
//			::FillRect(m_DC->m_hDC, &rc, hBrush);
	if(m_clipReg.ClipRectangle(rc))
		::FillRect(m_DC->m_hDC, &rc, hBrush);

	if(color != -1)
	{
		::SelectObject(m_DC->m_hDC, hBrold);
		::DeleteObject(hBrush);
	}
}

void CRealTimePlot::DrawClipText(long x, long y, CString &strText,int right_limit)
{
	// start position
	x = xPos(x), y = yPos(y);
	CRect rect(x, y, x, y);

	LPCSTR p = strText, s = NULL;
	int len = strText.GetLength();
	for (int i = 0; i < len; i++, p++)
	{
		SIZE sizeChar;
		::GetTextExtentPoint32(m_DC->m_hDC, p, 1, &sizeChar);
		rect.right += sizeChar.cx;
		if (i == 0) rect.bottom -= sizeChar.cy;
		
		if (-32768 <= rect.left && rect.left <= 32767)
		{
			if (m_clipReg.IsRectOnRect(rect))
			{
				if(right_limit > 1)
				{
					if(rect.left < xPos(right_limit))
						TextOut(m_DC->m_hDC, rect.left, rect.top, p, 1);
				}
				else
				{
					TextOut(m_DC->m_hDC, rect.left, rect.top, p, 1);
				}
			}
		}
		rect.left = rect.right;	
		if (rect.left > 32767) break;
	}	
}

void CRealTimePlot::DrawClipMarker(int x, int y, int mode, int n, COLORREF color)
{
	HBRUSH hBrush;
	HBRUSH hBrold;
	HPEN hCurPen;
	HPEN hOldPen;

	if(color != -1)
	{
		hCurPen = ::CreatePen(PS_SOLID, 0, color);
		hOldPen = (HPEN)::SelectObject(m_DC->m_hDC, hCurPen);

		hBrush = ::CreateSolidBrush(color);
		hBrold = (HBRUSH)::SelectObject(m_DC->m_hDC, hBrush);
	}

	if (m_bPrinting)	n *= m_nPrintScale;
    switch(mode)
	{
		case CROSS:
			DrawClipLine(x - n, y, x + n, y );
			DrawClipLine(x, y - n, x, y + n );
			break;

		case STAR:
			DrawClipLine(x - n, y, x + n, y );
			DrawClipLine(x - n / 2, (int)(y + n * sqrt(3.) / 2), x + n / 2, 
					(int)(y - n * sqrt(3.) / 2));
			DrawClipLine(x - n / 2, (int)(y - n * sqrt(3.) / 2), x + n / 2, 
					(int)(y + n * sqrt(3.) / 2));
			break;

		case FCIRCLE:
			//DrawFilledCircle(x, y, n);
			break;

		case FTRIANGLE:
		{
			POINT  p[3];
			p[0].x = x - n;
			p[0].y = (int)(y + n * sqrt(3.) / 3);
			p[1].x = x + n;
			p[1].y = (int)(y + n * sqrt(3.) / 3);
			p[2].x = x;
			p[2].y = (int)(y - 2 * n * sqrt(3.) / 3);

			// trans the orig to the bottom line
			for(int i=0; i<3;i++)
			{
				p[i].y -= (long)(n * sqrt(3.) / 3);
			}

			for(i=0; i<3;i++)
			{
				p[i].x = xPos(p[i].x);
				p[i].y = yPos(p[i].y);
			}

			::Polygon(m_DC->m_hDC, p, 3);
			break;
		}

		case REV_FTRIANGLE:
		{
			POINT  p[3];
			p[0].x = x - n;
			p[0].y = (int)(y + n * sqrt(3.) / 3);
			p[1].x = x + n;
			p[1].y = (int)(y + n * sqrt(3.) / 3);
			p[2].x = x;
			p[2].y = (int)(y - 2 * n * sqrt(3.) / 3);

			// trans the orig to the bottom line
			for(int i=0; i<3;i++)
			{
				p[i].y += (long)(2*n * sqrt(3.) / 3);
			}

			for(i=0; i<3;i++)
			{
				p[i].x = xPos(p[i].x);
				p[i].y = yPos(p[i].y);
			}
			::Polygon(m_DC->m_hDC, p, 3);
			break;
		}

		case XCROSS:
			DrawClipLine((int)(x + n * sqrt(2.) / 2),
					 (int)(y - n * sqrt(2.) / 2), 
					 (int)(x - n * sqrt(2.) / 2),
					 (int)(y + n * sqrt(2.) / 2));
			DrawClipLine((int)(x - n * sqrt(2.) / 2),
					 (int)(y - n * sqrt(2.) / 2), 
					 (int)(x + n * sqrt(2.) / 2),
					 (int)(y + n * sqrt(2.) / 2));
			break;

		case CIRCLE:
			//DrawCircle(x, y, n);
			break;

		case TRIANGLE:
			{
			POINT  p[3];
			p[0].x = x - n;
			p[0].y = (int)(y - n * sqrt(3.) / 3);
			p[1].x = x + n;
			p[1].y = (int)(y - n * sqrt(3.) / 3);
			p[2].x = x;
			p[2].y = (int)(y + 2 * n * sqrt(3.) / 3);

			// trans the orig to the bottom line
			for(int i=0; i<3;i++)
			{
				p[i].y -= (long)(2*n * sqrt(3.) / 3);
			}

			DrawClipLine(p[0].x, p[0].y, p[1].x, p[1].y);
			DrawClipLine(p[1].x, p[1].y, p[2].x, p[2].y);
			DrawClipLine(p[2].x, p[2].y, p[0].x, p[0].y);  
			}
			break;

		case REV_TRIANGLE:
			{
			POINT  p[3];
			p[0].x = x - n;
			p[0].y = (int)(y + n * sqrt(3.) / 3);
			p[1].x = x + n;
			p[1].y = (int)(y + n * sqrt(3.) / 3);
			p[2].x = x;
			p[2].y = (int)(y - 2 * n * sqrt(3.) / 3);

			// trans the orig to the bottom line
			for(int i=0; i<3;i++)
			{
				p[i].y += (long)(2*n * sqrt(3.) / 3);
			}

			DrawClipLine(p[0].x, p[0].y, p[1].x, p[1].y);
			DrawClipLine(p[1].x, p[1].y, p[2].x, p[2].y);
			DrawClipLine(p[2].x, p[2].y, p[0].x, p[0].y);  
			}
			break;

		case FSQUARE:
			//::Rectangle(m_DC->m_hDC, x - n, y - n, x + n, y + n);
			FillClipRectangle(x - n, y - n, x + n, y + n);
			break;

		case SQUARE:      
			DrawClipRect(CRect(x - n, y - n, x + n, y + n));
			break;

		case FDIAMOND:
		{
			POINT  p[4];
			p[0].x = x - n;
			p[0].y = y;
			p[1].x = x;
			p[1].y = y + n;
			p[2].x = x + n;
			p[2].y = y;
			p[3].x = x;
			p[3].y = y - n;
			
			for(int i=0; i<4;i++)
			{
				p[i].x = xPos(p[i].x);
				p[i].y = yPos(p[i].y);
			}
			::Polygon(m_DC->m_hDC, p, 4);
			break;
		}

		case DIAMOND:

			POINT  p[4];
			p[0].x = x - n;
			p[0].y = y;
			p[1].x = x;
			p[1].y = y + n;
			p[2].x = x + n;
			p[2].y = y;
			p[3].x = x;
			p[3].y = y - n;

			for(int i=0; i<4;i++)
			{
				p[i].x = xPos(p[i].x);
				p[i].y = yPos(p[i].y);
			}

			::MoveToEx(m_DC->m_hDC, p[0].x, p[0].y, NULL);
			::LineTo(m_DC->m_hDC, p[1].x, p[1].y);
			::LineTo(m_DC->m_hDC, p[2].x, p[2].y);
			::LineTo(m_DC->m_hDC, p[3].x, p[3].y);
			::LineTo(m_DC->m_hDC, p[0].x, p[0].y);
			break;
	}

	if(color != -1)
	{
		::SelectObject(m_DC->m_hDC, hBrold);
		::DeleteObject(hBrush);

		::SelectObject(m_DC->m_hDC, hOldPen);
		::DeleteObject(hCurPen);
	}
}

// convert client units into logical units (without offset)
long CRealTimePlot::xPos(int x)
{
	return MulDiv(x + m_x0, m_Scale, ZOOM_ONE) - m_xorg;
}

long CRealTimePlot::yPos(int y)
{
	return MulDiv(y + m_y0, m_Scale, ZOOM_ONE) - m_yorg;
}

// convert virtual logical units into client units
int CRealTimePlot::PosX(long x)
{
	return MulDiv(x, ZOOM_ONE, m_Scale) - m_x0;
}

int CRealTimePlot::PosY(long y)
{
	return MulDiv(y, ZOOM_ONE, m_Scale) + m_y0;
}
