#if !defined(__REALTIME_H__)
#define __REALTIME_H__

#include "graphics.h"
#include "ClassArr.h"
#include "Str.h"
#include "TaskChannel.h"
#include "ClipRegion.h"
#include "TaskPlotData.h"
#include <afxtempl.h>

#define ZOOM_ONE 1000

//////////////////////////////////////////////////////////////////////////////////
////// class CRealTime
class CRealTimePlot : public CGraphics
{
public:
	CTaskScreenArray	m_procs;

	int GetMaxChannel(int pindex) 
	{ 
		return m_procs.GetSize(pindex);
	}

	void AddValue(int pindex, int tindex, double time, int event)
	{ 
		CTaskChannel *aTask = m_procs.Get(pindex, tindex);
		aTask->AddValue(time, event); 
	}

public:
	virtual bool SetDataScale(int pindex, double xmin, double ymin, double xmax, double ymax);
	virtual void XAxis(int pindex);
	virtual void YAxis(int pindex);
	virtual void Grid(int pindex);

	bool InitComplete(int pindex, double maxtimetoshow);
	void Show();
	
	void SetTickLineWidth(int width)	{ m_wdTickLine = width; }
	int  GetTickLineWidth()				{ return m_wdTickLine; }
	void SetTickLineColor(COLORREF cr)	{ m_crTickLine = cr; }
	COLORREF  GetTickLineColor()		{ return m_crTickLine; }
	void SetTickBarHight(int high)		{ m_sHigh = high; }
	void Legend(COLORREF cr, int taskIndex,int index, const char* Name);
	void ChannelTitle(const char* Title, int index);

	void DrawLegend();
	int AddChannel(int pindex, const char* name, int priority, double period, double deadline, double wcet,double offset);
	void RemoveAll(int pindex);

//	bool RemoveALine(int pindex, int Index);
	void UpdateTimeRange(double& vtime);
	double GetCursorTime();
	int	 GetCursorTimeAndIndex(double&);
	void Redraw(HWND hWnd);
	RECT TickBarSize();
	void SetPrintTime(double time, int flag);
	void PrintTime();

	CRealTimePlot();
	~CRealTimePlot();

	CTaskChannel*	m_pCurLine;
	DWORD	m_nDataPerLine;
	bool	m_bSetingFailed;
	RECT	m_SRect;
	int		m_sHigh;
	double	m_nMaxTime;
	double	m_nCursorTime;
	DWORD	m_nTimes;
	CClipRegion m_clipReg;		// clipping region
	long  m_Scale;		// zoom for drawing function
	long  m_xorg, m_yorg;	// origine for drawing function
	CSize m_sizeDoc;	// document size
	long  m_x0, m_y0;	// document origin
	// GDI can only draw into valid logical units
	// so it is necessary to clip drawing object into valid range

	// convert client units into logical units (without offset)
	long xPos(int x);
	long yPos(int y);
	
	// convert virtual logical units into client units
	int PosX(long x);
	int PosY(long y);
	
	virtual void DrawClipLine(long x1, long y1, long x2, long y2, COLORREF color=-1);
	virtual void DrawClipRect(const CRect &rect, COLORREF color=-1);
	virtual void DrawClipText(long x, long y, CString &strText,int right_limit=0);
	virtual void FillClipRectangle(int x1, int y1, int x2,int y2, COLORREF color=-1);
	virtual void DrawClipMarker(int x, int y, int mode, int n=6, COLORREF color=-1);

	enum TimeTypes{MaxTime = 0, CursorTime};
	int		m_maxproc;
protected:
	HPEN	m_CurPen;
	HPEN	m_OldPen;
	char	m_sTempStr[32];

private:
	int		ForwardDraw(int nB, int nE);
	int		BackwardDraw(int nB, int nE);
	void	DrawXAxisTimeTicks(int x, int y, double& ticks);
	void	ShowTime(int x, int y, const char* Tag, double& cTime);
	void	DrawCurrentLine();
	void	DrawTickLine();
	
	double	CalXAxis(int pindex,double time);
	int		CalYAxis(int pindex,int index);

	DWORD	m_nP;
	int		m_wdTickLine;
	int		m_nStep;
	int		xb, yb, xe, ye; // save some time !!!
	COLORREF m_crTickLine;
};

#endif
