// TaskPlotWnd.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "TaskPlotWnd.h"
#include "RealTimePlot.h"
#include <limits.h>
#include <afxext.h>

/*
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
*/

#define g_ZoomFactor (1.25)

int MapMode = MM_LOMETRIC;

BEGIN_MESSAGE_MAP(TaskPlotWnd, CBigScrollView)
	//{{AFX_MSG_MAP(TaskPlotWnd)
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CBigScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CBigScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CBigScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(TaskPlotWnd, CBigScrollView)

void GetPrinterPageSize(CSize &pageSize)
{
	CPrintInfo PrintInfo;
//	pageSize = CSize(2100, 2960);		// default to A4
	pageSize = CSize(100, 100);		
	if (AfxGetApp()->GetPrinterDeviceDefaults(&PrintInfo.m_pPD->m_pd))
	{
		if (PrintInfo.m_pPD->CreatePrinterDC() != NULL)
		{
			if (PrintInfo.m_pPD->m_pd.hDC == NULL)
				PrintInfo.m_pPD->CreatePrinterDC();
		
			// attach printer dc and retreive docSize
			CDC dcPrint;
			dcPrint.Attach(PrintInfo.m_pPD->m_pd.hDC);
			pageSize.cx = 10 * dcPrint.GetDeviceCaps(HORZSIZE);
			pageSize.cy = 10 * dcPrint.GetDeviceCaps(VERTSIZE);
		}
	}
}

class CHandTracker : public CRectTracker
{
public:
	TaskPlotWnd *m_pView;
	CHandTracker(TaskPlotWnd *pView);
	virtual void DrawTrackerRect(LPCRECT lpRect, CWnd* pWndClipTo, CDC* pDC, CWnd* pWnd);
	virtual void OnChangedRect(const CRect& rectOld);
};

CHandTracker::CHandTracker(TaskPlotWnd *pView)
{
	ASSERT(pView != NULL);
	m_pView = pView;
	m_sizeMin = CSize(1, 1);
}

void CHandTracker::DrawTrackerRect(LPCRECT lpRect, CWnd* pWndClipTo, CDC* pDC, CWnd* pWnd)
{
	// draw nothing
}

void CHandTracker::OnChangedRect(const CRect& rectOld)
{
	CPoint point;
	// call the view to update scroll position
	point.x = m_rect.Width() - rectOld.Width();
	point.y = m_rect.Height() - rectOld.Height();
	m_pView->UpdateScrollPosition(point);
}

/////////////////////////////////////////////////////////////////////////////
// TaskPlotWnd

TaskPlotWnd::TaskPlotWnd()
{
	m_ZoomMin = 1;
	m_ZoomMax = LONG_MAX / m_Graph.m_sizeDoc.cy;
	m_Zoom = ZOOM_ONE;

	m_hCursor = NULL;
	m_TrackMode = 1;
	m_Graph.m_fontHeight = 0;

	for(int i=0; i<MAXPROC; i++)
		m_bInitDone[i] = 0;
}

TaskPlotWnd::~TaskPlotWnd()
{
	if (m_hCursor != NULL) DestroyCursor(m_hCursor);

	if (m_Graph.m_Font.m_hObject != NULL)
		m_Graph.m_Font.DeleteObject();

//	if (m_Graph.m_FontSmall.m_hObject != NULL)
//		m_Graph.m_FontSmall.DeleteObject();
}

void TaskPlotWnd::OnViewSetScaleToFit()
{
	CSize sizeTotal = m_TotalSize;

	// this function change the scale
	SetScaleToFitSize(m_TotalSize);

	// read the scale back
	OnZoom(MulDiv(m_Zoom, m_TotalSize.cx, sizeTotal.cx));
}

/////////////////////////////////////////////////////////////////////////////
// TaskPlotWnd printing

BOOL TaskPlotWnd::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void TaskPlotWnd::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void TaskPlotWnd::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// TaskPlotWnd message handlers
void TaskPlotWnd::OnDraw(CDC *dc)
{
//	CRect rect;
//	GetClientRect(&rect);

	// start to draw
	m_Graph.m_DC = dc;
	OnPrepareDC(dc);

	m_Graph.BeginDraw(dc->m_hDC);

	if(m_bInitDone[m_Graph.m_curproc] == 1) 
	{
		m_Graph.SetPaperScale(m_Graph.m_sizeDoc);
		
		m_Graph.SetScreenScale(m_Graph.m_curproc, CRect(0,0,m_Graph.m_sizeDoc.cx, m_Graph.m_sizeDoc.cy));

		m_Graph.DrawBkGround(NULL);
		//m_Graph.DrawBoundary(crBoundary, 1);

		m_Graph.Axes();
		m_Graph.Grid(m_Graph.m_curproc);
		m_Graph.Show();
		m_Graph.DrawLegend();	
	}
	
	m_Graph.EndDraw(dc->m_hDC);	
}

void TaskPlotWnd::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	CRect rect;
	if(m_bInitDone[m_Graph.m_curproc] != 1) return;
	
	CBigScrollView::OnPrepareDC(pDC, pInfo);
	if (pDC->IsPrinting())
	{
		int cx = 10 * pDC->GetDeviceCaps(HORZSIZE);
		int cy = 10 * pDC->GetDeviceCaps(VERTSIZE);
		rect.SetRect(0, 0, cx, -cy);

		m_Graph.m_xorg = 0;							// no offset for printing
		m_Graph.m_yorg = cy;
		m_Graph.m_Scale = ZOOM_ONE;					// no zoom for printing
	}
	else
	{
		GetLogClientRect(&rect);
		rect.OffsetRect(-m_Offset);			// clip rect in [lu]
		//rect.OffsetRect(0, m_Delta);		// correct window

		m_Graph.m_xorg = m_Offset.x;					// offset for drawing
		m_Graph.m_yorg = m_Offset.y + m_TotalSize.cy;	// origin bottom/left
		m_Graph.m_Scale = m_Zoom;					// zoom for drawing
	}

	rect.InflateRect(m_Delta, -m_Delta);
	m_Graph.m_clipReg = CClipRegion(rect);
	
	UpdateGDIObjects();
/*	
	if(m_hCursor == NULL)
	{
		m_hCursor = LoadCursor(NULL, MAKEINTRESOURCE(IDC_HANDCURSOR));
//		m_hCursor = LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_HANDCURSOR));
	}
*/
}

void TaskPlotWnd::UpdateGDIObjects()
{
	// calculate new font height
	int Height = m_Graph.m_sizeDoc.cx / 50;
	Height = MulDiv(Height, m_Graph.m_Scale, ZOOM_ONE);

	// if font valid and same font, return
	if (m_Graph.m_Font.m_hObject != NULL && Height == m_Graph.m_fontHeight) return;

	// delete old font
	if (m_Graph.m_Font.m_hObject != NULL)
		m_Graph.m_Font.DeleteObject();

//	if (m_Graph.m_FontSmall.m_hObject != NULL)
//		m_Graph.m_FontSmall.DeleteObject();

	// create fonts
	LOGFONT lf;	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = Height;
	lf.lfWeight = FW_NORMAL;
	lf.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(lf.lfFaceName, "Arial");
	VERIFY(m_Graph.m_Font.CreateFontIndirect(&lf));

/*	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = Height/2;
	lf.lfWeight = FW_EXTRALIGHT;
	lf.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(lf.lfFaceName, "Arial");
	VERIFY(m_Graph.m_FontSmall.CreateFontIndirect(&lf));
*/
	// save new font height
	m_Graph.m_fontHeight = Height;
}

void TaskPlotWnd::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
//	CBigScrollDoc* pDoc = GetDocument();
//	ASSERT_VALID(pDoc);
	//m_sizeDoc = pDoc->GetDocSize();
	m_Graph.m_x0 = 0;//m_Graph.m_sizeDoc.cx / 2;
	m_Graph.m_y0 = 0;//m_Graph.m_sizeDoc.cy / 2;

	SetScrollSizes(MapMode, m_Graph.m_sizeDoc);
	OnZoom(ZOOM_ONE);

}

void TaskPlotWnd::UpdateScrollPosition(CPoint point)
{
	if(m_bInitDone[m_Graph.m_curproc] == 0) return;
	
	CPoint pointCenter = GetLogClientCenter();
	if (m_nMapMode != MM_TEXT)
	{
		CWindowDC dc(NULL);
		dc.SetMapMode(m_nMapMode);
		dc.DPtoLP(&point);
	}
	pointCenter -= point;
	CenterOnPoint(pointCenter);
}

void TaskPlotWnd::OnZoom(long Zoom)
{
//	TRACE("Application instance handle is 0x%0X\n", ::AfxGetInstanceHandle());
	if (Zoom < m_ZoomMin)
	{
		MessageBeep(MB_ICONEXCLAMATION);
		Zoom = m_ZoomMin;
	}
	if (Zoom > m_ZoomMax)
	{
		MessageBeep(MB_ICONEXCLAMATION);
		Zoom = m_ZoomMax;
	}
	if (Zoom != m_Zoom)
	{
		m_Zoom = Zoom;
		CSize sizeTotal = m_Graph.m_sizeDoc;
		sizeTotal.cx = MulDiv(sizeTotal.cx, m_Zoom, ZOOM_ONE);
		sizeTotal.cy = MulDiv(sizeTotal.cy, m_Zoom, ZOOM_ONE);
		SetScrollSizes(MapMode, sizeTotal);
		//UpdateDisplayInfo();
	}
}

void TaskPlotWnd::OnViewZoomIn() 
{
	OnZoom(m_Zoom * g_ZoomFactor);
}

void TaskPlotWnd::OnViewZoomOut() 
{
	OnZoom(m_Zoom / g_ZoomFactor);
}

void TaskPlotWnd::OnViewZoomDefault() 
{
	// get client center in document units
	CRect rectClient;
	GetLogClientRect(&rectClient);
	CPoint pointCenter = rectClient.CenterPoint();
	pointCenter.x = MulDiv(pointCenter.x, ZOOM_ONE, m_Zoom);
	pointCenter.y = MulDiv(pointCenter.y, ZOOM_ONE, m_Zoom);
	
	// change zoom
	OnZoom(ZOOM_ONE);

	// calculate new center point with zoom
	pointCenter.x = MulDiv(pointCenter.x, m_Zoom, ZOOM_ONE);
	pointCenter.y = MulDiv(pointCenter.y, m_Zoom, ZOOM_ONE);

	// center on new point
	CenterOnPoint(pointCenter);
}

void TaskPlotWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	//TODO: fix
	//return;
	if (m_TrackMode == 1)
	{
		CHandTracker trackHand(this);
		CPoint saveCenter = m_Center;
		trackHand.TrackRubberBand(this, point);
		Invalidate();
		return;
	}
	else
	{
		CRectTracker tracker;
		if (tracker.TrackRubberBand(this, point)) 
		{
			CRect rectTracker = tracker.m_rect;
			rectTracker.NormalizeRect();
			if (rectTracker.Width() >= 8 * m_Delta)
			{
				long Zoom = m_Zoom;
				long saveZoom = m_Zoom;
								
				// get logical position from device position
				CPoint p1 = GetLogPosition(rectTracker.TopLeft());
				CPoint p2 = GetLogPosition(rectTracker.BottomRight());

				// get size of tracker rectangle
				rectTracker = CRect(p1, p2);
				CSize sizeTracker(abs(rectTracker.Width()), abs(rectTracker.Height()));
			
				// get center of tracker rectangle without zoom
				CPoint pointCenter = rectTracker.CenterPoint();
				pointCenter.x = MulDiv(pointCenter.x, ZOOM_ONE, m_Zoom);
				pointCenter.y = MulDiv(pointCenter.y, ZOOM_ONE, m_Zoom);
			
				// get client size
				CRect rectClient;
				GetLogClientRect(&rectClient);
				CSize sizeClient(rectClient.Width(), abs(rectClient.Height()));

				// calculate new zoom
				if (sizeClient.cy < MulDiv(sizeTracker.cy, sizeClient.cx, sizeTracker.cx))
					Zoom = MulDiv(Zoom, sizeClient.cy, sizeTracker.cy);
				else
					Zoom = MulDiv(Zoom, sizeClient.cx, sizeTracker.cx);

				// change zoom
				OnZoom(Zoom);

				// calculate new center point with new zoom
				if (m_Zoom != saveZoom)
				{
					pointCenter.x = MulDiv(pointCenter.x, m_Zoom, ZOOM_ONE);
					pointCenter.y = MulDiv(pointCenter.y, m_Zoom, ZOOM_ONE);
					// center on new point
					CenterOnPoint(pointCenter);
				}
			}
			return;
		}
	}
	CBigScrollView::OnLButtonDown(nFlags, point);
}

void TaskPlotWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
	//return;
	m_TrackMode = (m_TrackMode == 0) ? 1: 0;

	CBigScrollView::OnRButtonDown(nFlags, point);
}

BOOL TaskPlotWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	//If I call AfxGetInstanceHandle in PrepareDC, it causes a debug exception. 
	//but if I call it here it's OK.
	if(m_hCursor == NULL)
	{
		m_hCursor = LoadCursor(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDC_HANDCURSOR));
	}
	if (m_TrackMode == 1 && nHitTest == HTCLIENT)
	{
		if(m_hCursor)
			::SetCursor(m_hCursor);
		return 0;
	}
	return CBigScrollView::OnSetCursor(pWnd, nHitTest, message);
}

void TaskPlotWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
//	return;
	CBigScrollView::OnMouseMove(nFlags, point);
//	UpdateDisplayInfo();
}

void TaskPlotWnd::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
//	return;
	// only to update display infos
	CBigScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
//	UpdateDisplayInfo();
}

void TaskPlotWnd::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
//	return;
	// only to update display infos
	CBigScrollView::OnVScroll(nSBCode, nPos, pScrollBar);
//	UpdateDisplayInfo();
}

void TaskPlotWnd::OnSize(UINT nType, int cx, int cy) 
{
	// only to update display infos
	CBigScrollView::OnSize(nType, cx, cy);
//	UpdateDisplayInfo();
}

void TaskPlotWnd::Print()
{
//	HINSTANCE hinstance = AfxGetInstanceHandle();
	this->SendMessage(ID_FILE_PRINT_DIRECT,0,0);
}
